<?php

namespace Drupal\user_referral\Cache\Context;

use Drupal\Core\Cache\CacheableMetadata;
use Drupal\Core\Cache\Context\CacheContextInterface;
use Drupal\user_referral\Entity\UserReferralType;

/**
 * Defines the ReferralCookieCacheContext service, for "per referral cookie" caching.
 *
 * Cache context ID: 'user_referral_cookie'.
 */
class ReferralCookieCacheContext implements CacheContextInterface {

  /**
   * {@inheritdoc}
   */
  public static function getLabel() {
    return t("Referral Cookie");
  }

  /**
   * {@inheritdoc}
   */
  public function getContext() {
    $request = \Drupal::request();

    if ($cookie_data = $request->cookies->get(UserReferralType::COOKIE_NAME)) {
      // Retrieve referral info from the cookie
      $cookie = json_decode($cookie_data);

      if (!empty($cookie)) {
        $element['#default_value'] = [
          'referrer_id' => $cookie->uid,
          'referral_type' => $cookie->type,
        ];
        return $cookie->uid . ':' . $cookie->type;
      }
    }
    return '';
    
  }

  /**
   * {@inheritdoc}
   */
  public function getCacheableMetadata() {
    return new CacheableMetadata();
  }
}
