<?php

namespace Drupal\user_referral\Plugin\ExtraField\Display;

use Drupal\Core\Entity\ContentEntityInterface;
use Drupal\extra_field_plus\Plugin\ExtraFieldPlusDisplayFormattedBase;
use Drupal\user_referral\Entity\UserReferralType;

/**
 * Example Node Label Extra field with formatted output.
 *
 * @ExtraFieldDisplay(
 *   id = "user_referral_link",
 *   deriver = "\Drupal\user_referral\Plugin\Derivative\ReferralLinkDeriver",
 *   bundles = {
 *     "user.*",
 *   },
 *   visible = false
 * )
 */
class ReferralLink extends ExtraFieldPlusDisplayFormattedBase {

  /**
   * {@inheritdoc}
   */
  public function getLabel() {
    $settings = $this->getEntityExtraFieldSettings();
    return $settings['label'];
  }

  /**
   * {@inheritdoc}
   */
  public function getLabelDisplay() {
    $settings = $this->getEntityExtraFieldSettings();
    return $settings['label_display'];
  }

  /**
   * {@inheritdoc}
   */
  public function view(ContentEntityInterface $entity) {
    $user = \Drupal::currentUser();
    $build = [];
    $referral_type = $this->getReferralType();
    $type_id = $referral_type->id();
    if ($referral_type->isApplicable($entity) &&
      (
        $user->hasPermission("view $type_id user referral links") ||
        ($user->id() == $entity->id() && $user->hasPermission("view own $type_id user referral links"))
      )
    )  {
      $build = parent::view($entity);
    }
    return $build;
  }

  /**
   * {@inheritdoc}
   */
  public function viewElements(ContentEntityInterface $entity) {
    $settings = $this->getEntityExtraFieldSettings();

    $referral_type = $this->getReferralType();

    $element = [];

    if ($entity->id()) {
      $element = [
        '#type' => 'referral_link',
        '#title' => t('@type referral link', ['@type' => $referral_type->label()]),
        '#referral_link' => $referral_type->getReferralLink($entity),
        '#referral_type' => $referral_type,
        '#account' => $entity,
        '#attached' => [
          'library' => ['user_referral/drupal'],
        ],
      ];
    }

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  protected static function extraFieldSettingsForm(): array {
    $form['label'] = [
      '#type' => 'textfield',
      '#title' => t('Label'),
      '#required' => TRUE,
    ];

    $form['label_display'] = [
      '#type' => 'select',
      '#title' => t('Label display'),
      '#options' => [
        'above' => t('Above'),
        'inline' => t('Inline'),
        'hidden' => t('- Hidden -'),
        'visually_hidden' => t('- Visually Hidden -'),
      ],
      '#required' => TRUE,
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  protected function defaultFormValues(): array {
    $plugin_definition = $this->getPluginDefinition();

    $values = [
      'label' => $plugin_definition['label'],
      'label_display' => 'hidden',
    ];

    return $values;
  }

  public function getReferralType() {
    $parts = explode('__', $this->pluginId);
    return UserReferralType::load(array_pop($parts));
  }

}
